//////////////////////////////////////////////////////////////
//                                                          //
// PLAYSTATION CONTROLLER-FPGA PACMAN INTERFACE TOP         //
//                                                          //
// Version : 1.00                                           //
//                                                          //
// Copyright(c) 2003  Katsumi Degawa , All rights reserved  //
//                                                          //
// Important !                                              //
//                                                          //
// This program is freeware for non-commercial use.         //
// An author does no guarantee about this program.          //
// You can use this under your own risk.                    //
//                                                          //
//////////////////////////////////////////////////////////////

`timescale 100ps/10ps

module	psPAD_pac_interface(
		CLK_18M,
		RST,
		psCLK,
		psSEL,
		psTXD,
		psRXD,
		ps_PSW,
		CONF_SW,
		MODE_SW,
		MODE_EN,
		VIB_SW,
		VIB_DATA,
		LR_SW
		);

input	CLK_18M,RST;
input	CONF_SW,MODE_SW,MODE_EN,LR_SW;
input	[1:0]VIB_SW;
input	psRXD;
output	psCLK,psSEL,psTXD;

output	[7:0]ps_PSW;

wire	[7:0]RX_DATA_1,RX_DATA_2;
wire	[7:0]RX_DATA_3,RX_DATA_4,RX_DATA_5,RX_DATA_6;
wire	[7:0]XY_POS;
wire	CLK_250K;

	assign	ps_PSW[0] = XY_POS[0];	//		UP
	assign	ps_PSW[1] = XY_POS[1];	//		DOWN
	assign	ps_PSW[2] = XY_POS[2];	//		LEFT
	assign	ps_PSW[3] = XY_POS[3];	//		RIGHT
	assign	ps_PSW[4] = XY_POS[4];	//		CREDIT1 (MARU-BOTAN)		
	assign	ps_PSW[5] = XY_POS[5];	//		CREDIT2 (SANKAKU-BOTAN)
	assign	ps_PSW[6] = XY_POS[6];	//		START 1P	(START-BOTAN)
	assign	ps_PSW[7] = XY_POS[7];	//		START 2P (START & SHIKAKU-BOTAN)

input     [7:0]VIB_DATA;

CLK_250k_GEN	CLK_250k(
		CLK_18M,
		RST,
		CLK_250K
		);

psPAD_top	psPAD_top(
		CLK_250K,			//  MAIN CLK 250KHz
		RST,				//	MAIN RESET
		psCLK,				//	psCLK CLK OUT
		psSEL,				//	psSEL OUT
		psTXD,				//  psTXD OUT
		psRXD,				//	psRXD IN
		RX_DATA_1,			//	RX DATA 1 (8bit)
		RX_DATA_2,			//	RX DATA 2 (8bit)
		RX_DATA_3,			//	RX DATA 3 (8bit)
		RX_DATA_4,			//	RX DATA 4 (8bit)
		RX_DATA_5,			//	RX DATA 5 (8bit)
		RX_DATA_6,			//	RX DATA 6 (8bit) 
		CONF_SW,			//	Dualshook Config  ACTIVE-HI			
		MODE_SW,			//	Dualshook Mode Set DEGITAL PAD 0: ANALOG PAD 1:
		MODE_EN,			//	Dualshook Mode Control  OFF 0: ON 1:
		VIB_SW,				//	Vibration SW  VIB_SW[0] Small Moter OFF 0:ON  1:
							//				  VIB_SW[1] Bic Moter   OFF 0:ON  1(Dualshook Only)
		VIB_DATA			//	//	Vibration(Bic Moter)Data   8'H00-8'HFF (Dualshook Only)
		);

pac_interface	pac_interface(
		CLK_250K,
		RST,
		LR_SW,
		RX_DATA_1,
		RX_DATA_2,
		RX_DATA_3,
		RX_DATA_4,
		RX_DATA_5,	
		RX_DATA_6,	
		XY_POS
		);

endmodule

//  18.432MHz => 250KHz
module CLK_250k_GEN(
	CLK_18M,
	RST,
	CLK_250K
	); 

input	CLK_18M,RST;
output  CLK_250K;
reg		CLK_250K;
reg		[6:0]count;

	always@(posedge CLK_18M)
		if(RST == 0)begin
			count <= 0;
		end
		else begin
			if(count == 37)begin
				count <= 0;
			end
			else begin
				count <= count + 1;
			end
		end
	always@(posedge CLK_18M)
		if(RST == 0)begin
			CLK_250K <= 0;
		end
		else begin
			if(count == 0)begin
				CLK_250K <= ~CLK_250K;
			end
			else begin
				CLK_250K <= CLK_250K;				
			end
		end				
				
endmodule


module	pac_interface(
	CLK,
	RST,
	LR_SW,
	RX_DATA_1,
	RX_DATA_2,
	RX_DATA_3,
	RX_DATA_4,
	RX_DATA_5,	
	RX_DATA_6,	
	XY_POS
	);

input	CLK,RST;
input	LR_SW;
input	[7:0]RX_DATA_1,RX_DATA_2,RX_DATA_3,RX_DATA_4,RX_DATA_5,RX_DATA_6;
output	[7:0]XY_POS;
reg		[7:0]XY_POS;

	always@(posedge CLK or negedge RST )	
		if( RST == 0)begin
		   XY_POS <= 8'b11111111;
		end
		else begin
			if(RX_DATA_2[7]==0)begin					// START 2P ( SHIKAKU-BOTAN & START-BOTAN)
				if(RX_DATA_1[3]==0)
					XY_POS <= 8'b01111111;
				else
					XY_POS <= 8'b11111111;
			end
			else begin
				if((LR_SW == 0)&&								// RIGHT JOG CONTROL UP
					((RX_DATA_4 >= 8'h00 && RX_DATA_4 <= 8'h1F)&&
						(RX_DATA_3 >= 8'h20 && RX_DATA_3 <= 8'hDF)))
					XY_POS <= 8'b11111110;
				else if((LR_SW == 0)&&							// RIGHT JOG CONTROL DOWN
					((RX_DATA_4 >= 8'hE0 && RX_DATA_4 <= 8'hFF)&&
						(RX_DATA_3 >= 8'h20 && RX_DATA_3 <= 8'hDF)))
					XY_POS <= 8'b11111101;
				else if((LR_SW == 0)&&							// RIGHT JOG CONTROL LEFT
					((RX_DATA_4 >= 8'h20 && RX_DATA_4 <= 8'hDF)&&
						(RX_DATA_3 >= 8'h00 && RX_DATA_3 <= 8'h1F)))
					XY_POS <= 8'b11111011;
				else if((LR_SW == 0)&&							// RIGHT JOG CONTROL RIGHT
					((RX_DATA_4 >= 8'h20 && RX_DATA_4 <= 8'hDF)&&
						(RX_DATA_3 >= 8'hE0 && RX_DATA_3 <= 8'hFF)))
					XY_POS <= 8'b11110111;
				else if((LR_SW == 1)&&							// LEFT JOG CONTROL UP
					((RX_DATA_6 >= 8'h00 && RX_DATA_6 <= 8'h1F)&&
						(RX_DATA_5 >= 8'h20 && RX_DATA_5 <= 8'hDF)))
					XY_POS <= 8'b11111110;
				else if((LR_SW == 1)&&							// LEFT JOG CONTROL DOWN
					((RX_DATA_6 >= 8'hE0 && RX_DATA_6 <= 8'hFF)&&
						(RX_DATA_5 >= 8'h20 && RX_DATA_5 <= 8'hDF)))
					XY_POS <= 8'b11111101;
				else if((LR_SW == 1)&&							// LEFT JOG CONTROL LEFT
					((RX_DATA_6 >= 8'h20 && RX_DATA_6 <= 8'hDF)&&
						(RX_DATA_5 >= 8'h00 && RX_DATA_5 <= 8'h1F)))
					XY_POS <= 8'b11111011;
				else if((LR_SW == 1)&&							// LEFT JOG CONTROL RIGHT
					((RX_DATA_6 >= 8'h20 && RX_DATA_6 <= 8'hDF)&&
						(RX_DATA_5 >= 8'hE0 && RX_DATA_5 <= 8'hFF)))
					XY_POS <= 8'b11110111;
				else if(RX_DATA_2[5]==0)						// CREDIT1 (MARU-BOTAN)  
					XY_POS <= 8'b11101111;
				else if(RX_DATA_2[4]==0)						// CREDIT2 (SANKAKU-BOTAN)  
					XY_POS <= 8'b11011111;
				else if(RX_DATA_1[3]==0)						// START 1P	(START-BOTAN)
					XY_POS <= 8'b10111111;
				else 
					XY_POS <= 8'b11111111;
			end
		end									

endmodule

