//////////////////////////////////////////////////////////////
//                                                          //
// PLAYSTATION CONTROLLER(DUALSHOOK) INTERFACE TOP          //
//                                                          //
// Version : 1.00                                           //
//                                                          //
// Copyright(c) 2003  Katsumi Degawa , All rights reserved  //
//                                                          //
// Important !                                              //
//                                                          //
// This program is freeware for non-commercial use.         //
// An author does no guarantee about this program.          //
// You can use this under your own risk.                    //
//                                                          //
//////////////////////////////////////////////////////////////
`timescale 100ps/10ps		

module psPAD_top(
CLK_250K,       //  MAIN CLK 250KHz
RST,            //	MAIN RESET
psCLK,          //	psCLK CLK OUT
psSEL,          //	psSEL OUT
psTXD,          //  psTXD OUT
psRXD,          //	psRXD IN
RXDAT_1,        //	RX DATA 1 (8bit)
RXDAT_2,        //	RX DATA 2 (8bit)
RXDAT_3,        //	RX DATA 3 (8bit)
RXDAT_4,        //	RX DATA 4 (8bit)
RXDAT_5,        //	RX DATA 5 (8bit)
RXDAT_6,        //	RX DATA 6 (8bit) 
CONF_SW,        //	Dualshook Config  ACTIVE-HI			
MODE_SW,        //	Dualshook Mode Set DEGITAL PAD 0: ANALOG PAD 1:
MODE_EN,        //	Dualshook Mode Control  OFF 0: ON 1:
VIB_SW,         //	Vibration SW  VIB_SW[0] Small Moter OFF 0:ON  1:
                //				  VIB_SW[1] Bic Moter   OFF 0:ON  1(Dualshook Only)
VIB_DATA        //	Vibration(Bic Moter)Data   8'H00-8'HFF (Dualshook Only)
);

input	CLK_250K,RST;
input	psRXD;
output	psCLK;
output	psSEL;
output	psTXD;
output	[7:0]RXDAT_1;
output	[7:0]RXDAT_2;
output	[7:0]RXDAT_3;
output	[7:0]RXDAT_4;
output	[7:0]RXDAT_5;
output	[7:0]RXDAT_6;
input	[1:0]VIB_SW;
input	[7:0]VIB_DATA;
input	CONF_SW,MODE_SW,MODE_EN;

wire	RXWT,TXSET;
wire	[7:0]RXD_P;
wire	psTYPE;
wire	CLR;
wire	CONF_ENT;
wire	[3:0]TX_CONT;
wire	[7:0]TXD_SET;

TXD_COMMAND		TXD_COMMAND(
	CLK_250K,       //	CLOCK 250KHz
	RST,            //	RST   ACTIVE-LOW
	TXSET,          //	TXD-DATA  WRIGHT PULSE
	CLR,            //	SDI SEQ PULSE
	psTYPE,         //	DEGITAL PAD 0: ANALOG PAD 1:(RXD ANS)
	CONF_SW,        //  Dualshook Config  ACTIVE-HI
	MODE_SW,        //	Dualshook Mode Set DEGITAL PAD 0: ANALOG PAD 1:
	MODE_EN,        //	Dualshook Mode Control  OFF 0: ON 1:
	VIB_SW,         //	Vibration SW  VIB_SW[0] Small Moter OFF 0:ON  1:
                    //				  VIB_SW[1] Bic Moter   OFF 0:ON  1(Dualshook Only)
	VIB_DATA,       //	Vibration(Bic Moter)Data   8'H00-8'HFF (Dualshook Only)	
	TX_CONT,        //	SDI  BYTE-COUNTER
	RXD_P,          //	psRXD DATA(Para)
	TXD_SET,        //  psTXD SET DATA
/*	DUALSHOOK*/,    //  PAD TYPE    Dualshook 1: Other 0:
/*	CONF_LOG */,    //	Config Format DATA
	CONF_ENT,       //  Config Enter  Hi
/*	Err	*/          //  Err ANS       YES 1: NO 0:
	);

ps_cont_TXD	ps_cont_TXD(
	CLK_250K,
	RST,
	psCLK,
	RXWT,
/*	TXWT     */,
/*	TXEN     */,
	TXSET,
	TXD_SET,
	CLR,
	psSEL,
	psTXD,
/*	TXD_P     */,
	psTYPE,
/*	X_COUNT   */,
/*	COUNT_16M */,
	TX_CONT
	);

ps_cont_RXD	ps_cont_RXD(
	psCLK,
	RST,
	RXWT,
	psRXD,
	RXD_P,
	CONF_ENT,
	RXDAT_1,
	RXDAT_2,
	RXDAT_3,
	RXDAT_4,
	RXDAT_5,
	RXDAT_6,
	TX_CONT,
	psTYPE
	);

endmodule


module	TXD_COMMAND(
	CLK_250K,       //	CLOCK 250KHz
	RST,            //	RST   ACTIVE-LOW
	TXSET,			//	TXD-DATA  WRIGHT PULSE
	CLR,            //	SDI SEQ PULSE
	psTYPE,         //	DEGITAL PAD 0: ANALOG PAD 1:(RXD ANS)
	CONF_SW,        //  Dualshook Config  ACTIVE-HI
	MODE_SW,        //	Dualshook Mode Set DEGITAL PAD 0: ANALOG PAD 1:
	MODE_EN,        //	Dualshook Mode Control  OFF 0: ON 1:
	VIB_SW,         //	Vibration SW  VIB_SW[0] Small Moter OFF 0:ON  1:
                    //				  VIB_SW[1] Bic Moter   OFF 0:ON  1(Dualshook Only)
	VIB_DATA,       //	Vibration(Bic Moter)Data   8'H00-8'HFF (Dualshook Only)
	TX_CONT,        //	SDI  BYTE-COUNTER
	RXD_P,          //	psRXD DATA(Para)
	TXD_SET,        //  psTXD SET DATA
	DUALSHOOK,      //  PAD TYPE    Dualshook 1: Other 0:
	CONF_LOG,       //	Config Format DATA
	CONF_ENT,       //  Config Enter  Hi
	Err             //  Err ANS       YES 1: NO 0:
	);

input	CLK_250K,RST;
input	CLR;
input	TXSET;
input	psTYPE;
input	CONF_SW,MODE_SW,MODE_EN;
input	[1:0]VIB_SW;
input	[7:0]VIB_DATA;
input	[3:0]TX_CONT;
input	[7:0]RXD_P;
output	[7:0]TXD_SET;
output	CONF_ENT,DUALSHOOK,Err;
output	[5:0]CONF_LOG;

reg		[7:0]TXD_SET;
reg		CONF_ENT,DUALSHOOK,Err,CONF;
reg		[5:0]CONF_LOG;

		always@(posedge TXSET or negedge RST)
			if( RST == 0 )begin
				TXD_SET	<= 0;
				CONF_LOG	<= 0;
				Err		<= 1;
				CONF		<= 0;
			end
			else begin
				if( CONF_ENT )begin
					if( CONF_LOG == 6'b000000)begin
						if( psTYPE == 0)begin
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1: 	begin	TXD_SET <= 8'h42;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;
											  Err <= (RXD_P == 8'h41 ||
											  		  RXD_P == 8'h73 ||
													  RXD_P == 8'h53 )? 0:1; end
								3:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								4: 	begin	TXD_SET <= 8'h00;
								              CONF_LOG <= (Err == 0)? 6'b000001:6'b000000;
											  	Err <= Err;	end
							default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase				
						end
						else begin			
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1:	begin	TXD_SET <= 8'h42;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;
											  Err <= (RXD_P == 8'h41 ||
											  		 RXD_P == 8'h73 ||
													 RXD_P == 8'h53)? 0:1; end
								3: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								4:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								5:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								6: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								7: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								8: 	begin	TXD_SET <= 8'h00;
											  CONF_LOG <= (Err == 0)? 6'b000001:CONF_LOG;
											  	Err <= Err; end
						    default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase
						end
					end
					else if( CONF_LOG == 6'b000001)begin
						if( psTYPE == 0)begin
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1: 	begin	TXD_SET <= 8'h43;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								3:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= Err; end
								4: 	begin	TXD_SET <= 8'h00;
								              CONF_LOG <= (Err == 0)? 6'b000011:CONF_LOG;
											  	Err <= Err;	end
							default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase				
						end
						else begin			
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1:	begin	TXD_SET <= 8'h43;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								3: 	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= Err; end
								4:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								5:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								6: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								7: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								8: 	begin	TXD_SET <= 8'h00;
											  CONF_LOG <= (Err == 0)? 6'b000011:CONF_LOG;
											  	Err <= Err; end
						    default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase
						end
					end
					else if( CONF_LOG == 6'b000011)begin
						if( psTYPE == 0)begin
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1: 	begin	TXD_SET <= 8'h45;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;
											  Err <= (RXD_P == 8'hF3)? 0:1; end
								3:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								4: 	begin	TXD_SET <= 8'h00;
												if(Err)
													CONF_LOG <= 6'b111111;
												else begin
													if(CONF)
														CONF_LOG <= 6'b001111;
													else
														CONF_LOG <= 6'b000111;
												end
											  	Err <= Err;	end
							default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase				
						end
						else begin			
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1:	begin	TXD_SET <= 8'h45;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;
											  Err <= (RXD_P == 8'hF3)? 0:1; end
								3: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								4:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								5:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								6: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								7: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								8: 	begin	TXD_SET <= 8'h00;
												if(Err)
													CONF_LOG <= 6'b111111;
												else begin
													if(CONF)
														CONF_LOG <= 6'b001111;
													else
														CONF_LOG <= 6'b000111;
												end
											  	Err <= Err; end
						    default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase
						end
					end
					else if( CONF_LOG == 6'b000111)begin
						if( psTYPE == 0)begin
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1: 	begin	TXD_SET <= 8'h44;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								3:	begin	TXD_SET <= (MODE_SW == 0)? 8'h00:8'h01;	
												CONF_LOG <= CONF_LOG;Err <= Err; end
								4: 	begin	TXD_SET <= (MODE_EN == 0)? 8'h03:8'h00;
								              CONF_LOG <= (Err == 0)? 6'b001111:CONF_LOG;
											  	Err <= Err;	end
							default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase				
						end
						else begin			
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1:	begin	TXD_SET <= 8'h44;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								3: 	begin	TXD_SET <= (MODE_SW == 0)? 8'h00:8'h01;
												CONF_LOG <= CONF_LOG;Err <= Err; end
								4:	begin	TXD_SET <= (MODE_EN == 0)? 8'h03:8'h00;
												CONF_LOG <= CONF_LOG;Err <= Err; end
								5:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								6: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								7: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								8: 	begin	TXD_SET <= 8'h00;
											  CONF_LOG <= (Err == 0)? 6'b001111:CONF_LOG;
											  	Err <= Err; end
						    default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase
						end
					end
					else if( CONF_LOG == 6'b001111)begin
						if( psTYPE == 0)begin
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1: 	begin	TXD_SET <= 8'h4D;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								3:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								4: 	begin	TXD_SET <= 8'h01;
								              CONF_LOG <= (Err == 0)? 6'b011111:CONF_LOG;
											  	Err <= Err;	end
							default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase				
						end
						else begin			
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1:	begin	TXD_SET <= 8'h4D;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								3: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								4:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= Err; end
								5:	begin	TXD_SET <= 8'hFF;CONF_LOG <= CONF_LOG;Err <= Err; end
								6: 	begin	TXD_SET <= 8'hFF;CONF_LOG <= CONF_LOG;Err <= Err; end
								7: 	begin	TXD_SET <= 8'hFF;CONF_LOG <= CONF_LOG;Err <= Err; end
								8: 	begin	TXD_SET <= 8'hFF;
											  CONF_LOG <= (Err == 0)? 6'b011111:CONF_LOG;
											  	Err <= Err; end
						    default:begin	TXD_SET <= 8'hFF;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase
						end
					end
					else if( CONF_LOG == 6'b011111)begin
						if( psTYPE == 0)begin
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1: 	begin	TXD_SET <= 8'h43;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								3:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								4: 	begin	TXD_SET <= 8'h00;
								              		CONF_LOG <= (Err == 0)? 6'b111111:CONF_LOG;
											  	Err <= Err;
												CONF <= (CONF == 0)? 1:CONF;
											end
							default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase				
						end
						else begin			
							case(TX_CONT)
								0:	begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;Err <= 0;	end
								1:	begin	TXD_SET <= 8'h43;CONF_LOG <= CONF_LOG;Err <= Err; end
								2:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								3: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								4:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								5:	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								6: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								7: 	begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
								8: 	begin	TXD_SET <= 8'h00;
												CONF_LOG <= (Err == 0)? 6'b111111:CONF_LOG;
											  	Err <= Err; 
												CONF <= (CONF == 0)? 1:CONF;
												end
						    default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;Err <= Err; end
							endcase
						end
					end
					else begin
						Err <= 1;
						if( psTYPE == 0)begin
							case(TX_CONT)
								0: begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;	end
								1: begin	TXD_SET <= 8'h42;CONF_LOG <= CONF_LOG;	end
								2: begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
								3: begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
								4: begin	TXD_SET <= 8'h00;CONF_LOG <= 6'b111111;	end
						    default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
							endcase				
						end
						else begin			
							case(TX_CONT)
								0: begin	TXD_SET <= 8'h01;CONF_LOG <= CONF_LOG;	end
								1: begin	TXD_SET <= 8'h42;CONF_LOG <= CONF_LOG;	end
								2: begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
								3: begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
								4: begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
								5: begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
								6: begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
								7: begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
								8: begin	TXD_SET <= 8'h00;CONF_LOG <= 6'b111111;	end
							default:begin	TXD_SET <= 8'h00;CONF_LOG <= CONF_LOG;	end
							endcase
						end
					end
				end				
				else begin
					Err		 <= Err;
					if( psTYPE == 0)begin
						case(TX_CONT)
							0: TXD_SET <= 8'h01;
							1: TXD_SET <= 8'h42;
							2: TXD_SET <= 8'h00;
							3: begin
								CONF_LOG <= (RXD_P == 8'b00)? 6'b000000:CONF_LOG; 
								if(VIB_SW[0] == 1 && DUALSHOOK == 0)	
									TXD_SET <= 8'h40;
								else if(VIB_SW[0] == 1 && DUALSHOOK == 1)	
									TXD_SET <= 8'h01;
								else
									TXD_SET <= 8'h00;
							   end								
							4: begin
								if(VIB_SW[0] == 1 && DUALSHOOK == 0)	
									TXD_SET <= 8'h01;
								else if(VIB_SW[1] == 1 && DUALSHOOK == 1)	
									TXD_SET <= VIB_DATA;
								else
									TXD_SET <= 8'h00;
							   end									
						default:TXD_SET <= 8'h00;
						endcase				
					end
					else begin			
						case(TX_CONT)
							0: TXD_SET <= 8'h01;
							1: TXD_SET <= 8'h42;
							2: TXD_SET <= 8'h00;
							3: begin
								CONF_LOG <= (RXD_P == 8'b00)? 6'b000000:CONF_LOG; 
								if(VIB_SW[0] == 1 && DUALSHOOK == 0)	
									TXD_SET <= 8'h40;
								else if(VIB_SW[0] == 1 && DUALSHOOK == 1)	
									TXD_SET <= 8'h01;
								else
									TXD_SET <= 8'h00;
							   end
							4: begin
								if(VIB_SW[0] == 1 && DUALSHOOK == 0)	
									TXD_SET <= 8'h01;
								else if(VIB_SW[1] == 1 && DUALSHOOK == 1)	
									TXD_SET <= VIB_DATA;
								else
									TXD_SET <= 8'h00;
							   end	
							5: TXD_SET <= 8'h00;
							6: TXD_SET <= 8'h00;
							7: TXD_SET <= 8'h00;
							8: begin 
								TXD_SET <= 8'h00;
							   end
						default:TXD_SET <= 8'h00;
						endcase
					end
				end					
			end

		always@(posedge CLR or negedge RST)
			if( RST == 0 )begin
				CONF_ENT <= 0;
			end
			else begin
				if(CONF_SW == 1)begin
					case(CONF_LOG)
						6'b111111:CONF_ENT <= 0;
						default	 :CONF_ENT <= 1;
					endcase
				end
				else begin
					CONF_ENT <= 0;
				end
			end					

		always@(posedge CLK_250K or negedge RST)
			if( RST == 0 )begin
				DUALSHOOK <= 0;
			end
			else begin
				if((CONF_SW == 1)&&(CONF_ENT == 0)&&(Err == 0))begin
					DUALSHOOK <= 1;
				end
				else begin
					DUALSHOOK <= 0 ;
				end
			end	
endmodule


module ps_cont_RXD(
	CLK,
	RST,
	RXWT,
	RXD,
	RXD_P,
	CONF_ENT,
	RX_DATA_1,
	RX_DATA_2,
	RX_DATA_3,
	RX_DATA_4,
	RX_DATA_5,
	RX_DATA_6,
	TX_CONT,
	TYPE
	);

input	CLK,RST,RXWT;
input	RXD;
input	CONF_ENT;
input	[3:0]TX_CONT;
output	[7:0]RXD_P;
output	[7:0]RX_DATA_1,RX_DATA_2;
reg		[7:0]RX_DATA_1,RX_DATA_2;
output	[7:0]RX_DATA_3,RX_DATA_4,RX_DATA_5,RX_DATA_6;
reg		[7:0]RX_DATA_3,RX_DATA_4,RX_DATA_5,RX_DATA_6;
output	TYPE;
reg		TYPE;

	ps_cont_rx	ps_cont_rx(CLK,RST,RXWT,RXD_P,RXD);

	always@(negedge RXWT or negedge RST)
		if(RST == 0)begin
			RX_DATA_1 <= 8'hFF;
			RX_DATA_2 <= 8'hFF;
		end
		else begin
			if(TX_CONT == 3 && CONF_ENT == 0)begin
				RX_DATA_1 <= RXD_P;
				RX_DATA_2 <= RX_DATA_2;
			end
			else if(TX_CONT == 4 && CONF_ENT == 0)begin
				RX_DATA_1 <= RX_DATA_1;
				RX_DATA_2 <= RXD_P;
			end
			else begin
				RX_DATA_1 <= RX_DATA_1;
				RX_DATA_2 <= RX_DATA_2;
			end
		end
	always@(negedge RXWT or negedge RST)
		if(RST == 0)begin
			RX_DATA_3 <= 8'h7F;
			RX_DATA_4 <= 8'h7F;
			RX_DATA_5 <= 8'h7F;
			RX_DATA_6 <= 8'h7F;
		end
		else begin
			if(TYPE)begin
				if(TX_CONT == 5 && CONF_ENT == 0)begin
					RX_DATA_3 <= RXD_P;
					RX_DATA_4 <= RX_DATA_4;
					RX_DATA_5 <= RX_DATA_5;
					RX_DATA_6 <= RX_DATA_6;
				end
				else if(TX_CONT == 6 && CONF_ENT == 0)begin
					RX_DATA_3 <= RX_DATA_3;
					RX_DATA_4 <= RXD_P;
					RX_DATA_5 <= RX_DATA_5;
					RX_DATA_6 <= RX_DATA_6;
				end
				else if(TX_CONT == 7 && CONF_ENT == 0)begin
					RX_DATA_3 <= RX_DATA_3;
					RX_DATA_4 <= RX_DATA_4;
					RX_DATA_5 <= RXD_P;
					RX_DATA_6 <= RX_DATA_6;
				end
				else if(TX_CONT == 8 && CONF_ENT == 0)begin
					RX_DATA_3 <= RX_DATA_3;
					RX_DATA_4 <= RX_DATA_4;
					RX_DATA_5 <= RX_DATA_5;
					RX_DATA_6 <= RXD_P;
				end
			end
			else begin
				RX_DATA_3 <= RX_DATA_3;
				RX_DATA_4 <= RX_DATA_4;
				RX_DATA_5 <= RX_DATA_5;
				RX_DATA_6 <= RX_DATA_6;
			end
		end

	always@(negedge RXWT or negedge RST)
		if(RST == 0)begin
			TYPE <= 0;
		end
		else begin
			if(TX_CONT == 1 && RXD_P == 8'h41)begin
				TYPE <= 0;				//  DEGITAL PAD
			end
			else if(TX_CONT == 1 && ((RXD_P == 8'h53)||(RXD_P == 8'h73)||(RXD_P == 8'hF3)))begin
				TYPE <= 1;				//	ANALOG PAD
			end
			else begin
				TYPE <= TYPE;
			end
		end

endmodule


module ps_cont_TXD(
	CLK,
	RST,
	CCLK,
	RXWT,
	TXWT,
	TXEN,
	TXSET,
	TXD_SET,
	CLR,SEL,
	TXD,
	TXD_P,
	TYPE,
	X_COUNT,
	COUNT_16M,
	TX_CONT
	);

input	CLK,RST;
input	TYPE;
input	[7:0]TXD_SET;
output	TXSET;
output	CCLK;
output	RXWT,TXWT,TXEN;
output	TXD;
output	[7:0]TXD_P;
output	CLR;
output	[4:0]X_COUNT;
output	[11:0]COUNT_16M;
output	[3:0]TX_CONT;
output	SEL;

TX_count		TX_count	(CLK,RST,X_COUNT,CLR,TYPE,TX_CONT);
seq_pls			seq_16msec	( CLK,RST,COUNT_16M,CLR);
ps_pulse_gen	ps_pulse_gen( CLK,RST,CLR,SEL,X_COUNT,CCLK,RXWT,TXWT,TXEN,TXSET);
ps_SEL			ps_SEL		(CLK,RST,TYPE,CLR,COUNT_16M,SEL);
ps_cont_tx		TX			( CLK,RST,TXWT,TXEN,TXD_SET,TXD);
ps_cont_rx		TX_test		(CCLK,RST,RXWT,TXD_P,TXD);

endmodule


module	ps_SEL(
	CLK,
	RST,
	TYPE,
	CLR,
	COUNT_16M,
	SEL
	);

input	CLK,RST;
input	CLR;
input	TYPE;
input	[11:0]COUNT_16M;
output	SEL;
reg		SEL;
			
	always@(posedge CLK or negedge RST)
		if( RST == 0 )
			SEL <= 1;
		else begin	
			if(CLR == 1)
				SEL <= 0;
			else if((TYPE == 0)&&(COUNT_16M == 158))
				SEL <= 1;
			else if((TYPE == 1)&&(COUNT_16M == 286))
				SEL <= 1;
			else 
				SEL <= SEL;
		end

endmodule


module	TX_count(
	CLK,
	RST,
	X_COUNT,
	CLR,
	TYPE,
	TX_CONT
	);

input	CLK,RST;
input	TYPE;
input	CLR;
input	[4:0]X_COUNT;
output	[3:0]TX_CONT;
reg		[3:0]TX_CONT;
			
	always@(negedge CLK or negedge RST)
		if( RST == 0 )
			TX_CONT <= 8;
		else begin					
			if(((TX_CONT >= 5)&&(TYPE == 0)) || CLR == 1)
				TX_CONT <= 0;
			if(((TX_CONT >= 9)&&(TYPE == 1)) || CLR == 1)
				TX_CONT <= 0;
			if(X_COUNT == 31)
				TX_CONT <= TX_CONT +1;
		end 

endmodule


module ps_cont_tx(
	CLK,
	RST,
	WT,
	EN,
	TXD_P,
	TXD
	);

input	CLK,RST;
input	WT,EN;
input	[7:0]TXD_P;
output	TXD;
reg		TXD;
reg		[7:0]ps;


	always@(negedge CLK or negedge RST)
		if(RST == 0)begin 
			TXD <= 1;
			ps  <= 0;
		end
		else begin
			if(WT)begin
				ps  <= TXD_P;
			end
			else begin
				if(EN)begin
					TXD <= ps[0];
					ps  <= { 1'b1, ps[7:1]};
				end
				else begin
					TXD <= 1'd1;
					ps  <= ps;
				end
			end
		end 	

endmodule


module ps_cont_rx(
	CLK,
	RST,
	WRT,
	RXD_P,
	RXD
	);

input	CLK,RST,WRT;
input	RXD;
output	[7:0]RXD_P;
reg		[7:0]RXD_P;
reg		[7:0]sp;

	always@(posedge CLK or negedge RST)
		if(! RST)	sp <= 0;
		else		sp <= { RXD, sp[7:1]};
	always@(posedge WRT or negedge RST)
		if(! RST)	RXD_P <= 0;
		else		RXD_P <= sp;

endmodule


module seq_pls(
	CLK,
	RST,
	COUNT_16M,
	CLR
	);

input	CLK,RST;
output	[11:0]COUNT_16M;
reg		[11:0]COUNT_16M;
output	CLR;
reg		CLR;

	parameter COUNT = 4000;

	always@(posedge CLK or negedge RST)
		if(RST == 0)begin
			COUNT_16M	<= COUNT-2;
			CLR			<= 0;
		end
		else begin	
			if(COUNT_16M == COUNT-1)begin
				COUNT_16M	<= 0;
				CLR			<= 1;
			end
			else begin
				COUNT_16M	<= COUNT_16M +1;
				CLR			<= 0;
			end
		end

endmodule


module ps_pulse_gen(
	CLK,
	RST,
	C_RST,
	SEL,
	COUNT,
	CCLK,
	RXWT,
	TXWT,
	TXEN,
	TXSET
	);

input	CLK,RST,C_RST,SEL;
output	[4:0]COUNT;
reg		[4:0]COUNT;
output	CCLK,RXWT,TXWT,TXEN,TXSET;
reg		CCLK_1,RXWT,TXWT,TXEN,TXSET;



	always@(posedge CLK or negedge RST)
		if(RST == 0)begin
			COUNT	<= 0;
		end
		else begin
			if(C_RST)begin
				COUNT	<= 0;			
			end
			else begin	
				if(SEL == 0)begin
					COUNT	<= COUNT+1;
				end
				else begin
					COUNT	<= COUNT;
				end
			end
		end

	always@(posedge CLK or negedge RST)
		if(RST == 0)begin
			CCLK_1	<= 1;
		end
		else begin
			case(COUNT)
				12:	CCLK_1 <= 0;
				20:	CCLK_1 <= 1;
				default:;
			endcase
		end
	assign CCLK = CCLK_1 | CLK;

	always@(posedge CLK or negedge RST)
		if(RST == 0)begin
			RXWT	<= 0;
		end
		else begin
			case(COUNT)
				21:	RXWT <= 1;
				23:	RXWT <= 0;
				default:;
			endcase
		end
	always@(posedge CLK or negedge RST)
		if(RST == 0)begin
			TXWT	<= 0;
		end
		else begin
			case(COUNT)
				 9:	TXWT <= 1;
				11:	TXWT <= 0;
				default:;
			endcase
		end
	always@(posedge CLK or negedge RST)
		if(RST == 0)begin
			TXSET	<= 0;
		end
		else begin
			case(COUNT)
				 6:	TXSET <= 1;
				 8:	TXSET <= 0;
				default:;
			endcase
		end
	always@(posedge CLK or negedge RST)
		if(RST == 0)begin
			TXEN	<= 0;
		end
		else begin
			case(COUNT)
				12:	TXEN <= 1;
				21:	TXEN <= 0;
				default:;
			endcase
		end
endmodule

