/////////////////////////////////////////////////////////////////////////////
// FPGA PACMAN TOP
//
// Version : beta2
//
// Copyright(c) 2002,2003 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// This program is freeware for non-commercial use. 
// An author does no guarantee about this program.
// You can use this under your own risk.
//
// 2003. 5. 8 Altera FPGA modification by Katsumi Degawa
// 2003. 5.15 PLAYSTATION CONTROLLER(DUALSHOOK)INTERFACE 
//            modification by Katsumi Degawa
//
/////////////////////////////////////////////////////////////////////////////

`define  dualshook

module top(
// Universal Video I/F
	VCLK,
	VGA_R,VGA_G,VGA_B,
	n_HSYNC,n_VSYNC,
//CPU interface
//ROM interface
	RA,RD,RCS,ROE,RWE,
//Sound
	DAC_L,DAC_R,
//Contyroll switches
	PSW,
// 18.432MHz general CLOCK input
	CLK18M432,
// PS_PAD interface
	psCLK,
	psSEL,
	psTXD,
	psRXD
);
/////////////////////////////////////////////////////////////////////////////
// module I/O assign
/////////////////////////////////////////////////////////////////////////////
output DAC_L,DAC_R;

input [3:0] PSW;
input CLK18M432; // 18.432MHz

// Universal Video I/F
input  VCLK;
output [4:0] VGA_R , VGA_G , VGA_B;
output n_HSYNC,n_VSYNC;

// code / pattern ROM interface
output [18:0] RA;
inout [7:0] RD;
output RCS,ROE,RWE;

// PS_PAD interface
input	psRXD;
output	psTXD,psCLK,psSEL;

//CPU interface
//input [15:0] XA;
//input [7:0] XD;
//inout XMREQ,XIORQ,XM1,XRD,XWR;
//output XCLK;

/////////////////////////////////////////////////////////////////////////////
// clock generator
/////////////////////////////////////////////////////////////////////////////
wire CLK18_432M = CLK18M432;
wire CLK12_288M;
wire CLK12_288M_240;
wire CLK6_144M;

clock BASE_CLOCK(
	.CLKIN(CLK18_432M),
	.CLK_2_3(CLK12_288M),
	.CLK_1_6(CLK6_144M),
	.CLK_1_6_n()
);
assign CLK12_288M_240 = ~CLK12_288M;

// base & video pixel clock
wire pixel_clk;
BUFG PIXEL_CLOCK(.I(CLK6_144M),.O(pixel_clk));

// ticker , clock  counter (PCLK = 18.432MHz)
// ticker[16] = @140Hz
// ticker[24] = @1Hz
reg  [39:0] ticker;
wire CLK1Hz;

assign CLK1Hz =ticker[23]; // @1Hz
//always @(posedge PCLK)
always @(posedge CLK18_432M)
begin
		ticker <= ticker + 1;   // 18.432MHz
end

/////////////////////////////////////////////////////////////////////////////
// Z80 core
/////////////////////////////////////////////////////////////////////////////

wire [7:0] ZDO,ZDI;
//output
wire [15:0] ZA;
wire ZMREQ,ZIORQ,ZM1,ZRD,ZWR,ZRFSH;
wire ZWO;
//input
wire Z80CLK2,Z80CLK,ZINT,ZNMI,ZRESET,ZWAIT;

Z80IP CPU(
	.CLK2X(Z80CLK2),
	.CLK(Z80CLK),
	.RESET_N(ZRESET),
	.INT_N(ZINT),
	.NMI_N(ZNMI),
	.A(ZA),
	.DOUT(ZDO),
	.DIN(ZDI),
	.M1_N(ZM1),
	.MREQ_N(ZMREQ),
	.IORQ_N(ZIORQ),
	.RD_N(ZRD),
	.WR_N(ZWR),
	.WAIT_N(ZWAIT),
	.BUSWO(ZWO),
	.RFSH_N(ZRFSH),
	.HALT_N()
);
assign ZRESET = PSW[0]|PSW[1]|PSW[2]|PSW[3];
assign ZNMI = 1'b1;

/////////////////////////////////////////////////////////////////////////////
// address decoder
/////////////////////////////////////////////////////////////////////////////

wire flip;
wire in0,in1,dsw1,dsw2;
wire wr0,wr1;
wire snd_on;
wire csxyram , csvram;
wire csrom;
adec pacman_adec(
// inputs
	.PCLK(pixel_clk),
	.A(ZA),
	.n_MREQ(ZMREQ),
	.D0I(ZDO[0]),
	.n_RD(ZRD),
	.n_RFSH(ZRFSH),
	.RST(1'b0),
// decode output
	.ROM(csrom),
	.RAM(csvram),
	.IN0(in0),
	.IN1(in1),
	.DSW1(dsw1),
	.DSW2(dsw2),
	.WR0(wr0),
	.WR1(wr1),	// 5050-505f.W sound frequency   / volume
	.WR2(csxyram),	// 5060-506f.W object X,Y RAM
	.WDT(),	// 50c0-50ff.W Watchdog timer reset
// latched register
	.IRQ(irqen),	// 5000.W      IRQ enable (clear)
	.SOUND_ON(snd_on),// 5001.W      sound enable
	.FLIP(flip),	// 5003.W      flip
	.LAMP1P(),	// 5004.W      1P start lamp
	.LAMP2P(),	// 5005.W      2P start lamp
	.COINLK(),	// 5006.W      coin lockout
	.COINUP()	// 5007.W      coin countup
);

/////////////////////////////////////////////////////////////////////////////
// CPU PROM to shared ROM interface
/////////////////////////////////////////////////////////////////////////////

wire [18:0] cpu_a;
reg [7:0] cpu_rd;
wire [7:0] rom_do = csrom ? cpu_rd : 8'h00;
assign cpu_a[15:0]  = ZA;
assign cpu_a[18:16] = 0;

/////////////////////////////////////////////////////////////////////////////
// paPAD Interface
/////////////////////////////////////////////////////////////////////////////
//PS_PAD Interface
wire	[7:0]ps_PSW;
`ifdef dualshook
reg	    erase,play;
reg     tunnel_x,tunnel_y;
wire	[1:0]VIB_SW = {erase&play,tunnel_x&tunnel_y&play};
`else
wire	[1:0]VIB_SW = 2'b00;
`endif
wire    [7:0]VIB_DAT=8'hFF;

psPAD_pac_interface	ps_PAD(
	CLK18_432M,
	ZRESET,
	psCLK,
	psSEL,
	psTXD,
	psRXD,
	ps_PSW,
	1'b1,
	1'b1,
	1'b0,
	VIB_SW,
	VIB_DAT,
	1'b0
	);

/////////////////////////////////////////////////////////////////////////////
// controller IN / DIP switch I/F
/////////////////////////////////////////////////////////////////////////////
//
//
// controller IN / DIP switch I/F
//
wire L1 = PSW[2] & ps_PSW[2];
wire R1 = PSW[3] & ps_PSW[3];
wire U1 = PSW[0] & ps_PSW[0];
wire D1 = PSW[1] & ps_PSW[1];
wire S1 = (PSW[2]|~PSW[3])& ps_PSW[6];
wire S2 = (PSW[3]|~PSW[2])& ps_PSW[7];
wire C1 = (PSW[2]|PSW[3]|PSW[0]|~PSW[1])& ps_PSW[4];
wire C2 = (PSW[2]|PSW[3]|PSW[1]|~PSW[0])& ps_PSW[5];

//wire L1 = PSW[2];
//wire R1 = PSW[3];
//wire U1 = PSW[0];
//wire D1 = PSW[1];
//
wire L2 = L1;
wire R2 = R1;
wire U2 = U1;
wire D2 = D1;

//wire S1 = L1|~R1;
//wire S2 = R1|~L1;
//wire C1 = L1|R1|U1|~D1;
//wire C2 = L1|R1|~U1|D1;
wire [7:0] inp_do;
inport pacman_inport(
// inputs
	.LEFT1(L1),
	.RIGHT1(R1),
	.UP1(U1),
	.DOWN1(D1),
	.LEFT2(L2),
	.RIGHT2(R2),
	.UP2(U2),
	.DOWN2(D2),
	.START1(S1),
	.START2(S2),
	.COIN1(C1),
	.COIN2(C2),
	.CREDIT(1'b1),
	.TEST(1'b1),
	.TABLE(1'b1),
//enables
	.IN0(in0),
	.IN1(in1),
	.DSW1(dsw1),
	.DSW2(dsw2),
//output
	.DO(inp_do)	
);

/////////////////////////////////////////////////////////////////////////////
// VIDEO hardware interface
/////////////////////////////////////////////////////////////////////////////

wire [8:0] hcnt;
wire hsync,vsync;
wire [7:0] vdo;
wire vblank;
wire [2:0] red;
wire [2:0] green;
wire [1:0] blue;
reg [7:0] pd;   //pattern ROM data
wire [12:0] pa; //patterm ROM address

// viode & work RAM I/F
wire [11:0] AB;
wire [7:0] DB_I;
wire tile_we , col_we , wram_we;
wire tile_cs , col_cs , wram_cs;
// line ram
wire [7:0] lram_a;
wire [7:0] lram_di;
wire [7:0] lram_do;
wire kram_cs,lram_we;
// object position ram
wire [7:0] pram_di,pram_do;
wire pram_we;
wire [7:0] clut_a;
wire [3:0] clut_d;
wire [3:0] pal_a;

video vh(
	.A(ZA[11:0]),
	.csvram(csvram),
	.csxyram(csxyram),
	.n_rd(ZRD),
	.di(ZDO),
	.do(vdo),
	.WAIT_N(ZWAIT),
	.pclk(pixel_clk),
	.flip(flip),
	.vb(vblank),
	.hc(hcnt),
	.hs(hsync),
	.vs(vsync),
//	.r(red),
//	.g(green),
//	.b(blue),
	.CA(pa),
	.CD(RD),
// CPU & VIDEO memory bus , & sound timming address
	.AB(AB), 
// VIDEO memory bus & controll
	.DB_I(DB_I),
	.TRAM_WE(tile_we),
	.TRAM_CS(tile_cs),
	.CRAM_WE(col_we),
	.CRAM_CS(col_cs),
	.WRAM_WE(wram_we),
	.WRAM_CS(wram_cs),
// line ram
	.LRAM_A(lram_a),
	.LRAM_DI(lram_di),
	.LRAM_DO(lram_do),
	.LRAM_CS(lram_cs),
	.LRAM_WE(lram_we),
// object position ram
	.PRAM_DI(pram_di),
	.PRAM_DO(pram_do),
	.PRAM_WE(pram_we),
// CLUT ROM
	.CLUT_A(clut_a),
	.CLUT_DI(clut_d),
// PaletteROM
	.PAL_A(pal_a)
);

/////////////////////////////////////////////////////////////////////////////
// CPU clock assign
/////////////////////////////////////////////////////////////////////////////

assign Z80CLK2 = pixel_clk;         // 6MHz
BUFG ZCLK(.I(~hcnt[0]),.O(Z80CLK)); // 3MHz

assign ZWAIT = ~(csvram & hcnt[1] & ~ZRD); // wait controll

/////////////////////////////////////////////////////////////////////////////
// IRQ & IRQ vector
/////////////////////////////////////////////////////////////////////////////

wire [7:0] irq_do;
wire vector_enable;
irq pacman_irqvector(
//inputs
	.n_M1(ZM1),
	.n_IORQ(ZIORQ),
	.n_WR(ZWR),
	.IRQEN(irqen),
	.IRQTRG(vblank),
	.DI(ZDO),
//outputs
	.n_IRQ(ZINT),
	.DO(irq_do),
	.DE(vector_enable)
);

/////////////////////////////////////////////////////////////////////////////
// sound
/////////////////////////////////////////////////////////////////////////////

wire [9:0] wave_out;
wire [8:0] wave_a;
wire [3:0] wave_d;
sound sound(
	.AB(ZA[3:0]),
//	.AB(AB[3:0]),
	.DI(ZDO[3:0]),
	.WR0(wr0),
	.WR1(wr1),
	.PCLK(pixel_clk),
	.SOUND_ON(snd_on),
	.HC(hcnt[5:0]),
//
	.DAO(wave_out),
	.WA(wave_a),
	.WDI(wave_d)
);

/////////////////////////////////////////////////////////////////////////////
// memory 
/////////////////////////////////////////////////////////////////////////////
wire [7:0] pal_d;

Amemory ALTERAMemory(
  .CLK_12M(CLK12_288M),
  .AB(AB),.DB_I(ZDO),.DB_O(DB_I),
  .TILE_CS(tile_cs),.TILE_WE(tile_we),
  .COL_CS(col_cs),.COL_WE(col_we),
  .WORK_CS(wram_cs),.WORK_WE(wram_we),
  .PWE(pram_we),.PDI(pram_do),.PDO(pram_di),
  .LA(lram_a),.LDO(lram_di),.LDI(lram_do),.LCS(lram_cs),.LWE(lram_we),
//  SA,S0DI,S0DO,S1DI,S1DO,SWE,
  .WA(wave_a),.WDO(wave_d),
  .CLUTA(clut_a),.CLUT_DO(clut_d),
  .PAL_A(pal_a),.PAL_DO(pal_d)
);
// R,G,B output signal
assign {blue,green,red} = pal_d;

/////////////////////////////////////////////////////////////////////////////
// external CPU code / patterm ROM interface
//
// interleave CPU address & video pattern address
/////////////////////////////////////////////////////////////////////////////

reg [18:0] rom_a;
always @(posedge pixel_clk)
begin
	if(~hcnt[0])
	begin
		// latch CPU data & switch video address
		cpu_rd <= RD;
		rom_a  <= { 6'b000011 ,  pa}; //06000-07fff
	end else begin
		// latch pixel data & switch cpu address
		pd    <= RD;
		rom_a <= cpu_a;
	end
end
assign RA  = rom_a;
assign RD  = 8'bzzzzzzzz;
assign RCS = 1'b0;
assign RWE = 1'b1;
assign ROE = 1'b0;

/////////////////////////////////////////////////////////////////////////////
// data bus multiplexer
/////////////////////////////////////////////////////////////////////////////

assign ZDI = vdo | rom_do | inp_do | irq_do;

/////////////////////////////////////////////////////////////////////////////
// Video encoder
/////////////////////////////////////////////////////////////////////////////

// video clock
wire video_clock;
IBUFG video_clock_i(.I(VCLK),.O(video_clock));

wire hs_out , vs_out;
wire hs_tri , vs_tri;
wire [7:0] rgb_in = { red,green,blue };
wire [4:0] r_out,g_out,b_out;
wire [4:0] r_tri,g_tri,b_tri;
venc pacman_video_encoder(
	.pclk(pixel_clk),
	.dclk(CLK12_288M),
	.vclk(video_clock),
	.rgb8_in(rgb_in),
	.hs_in(hsync),
	.vs_in(vsync),
//
	.r_out(r_out),
	.r_tri(r_tri),
	.g_out(g_out),
	.g_tri(g_tri),
	.b_out(b_out),
	.b_tri(b_tri),
	.hs_tri(hs_tri),
	.hs_out(hs_out),
	.vs_tri(vs_tri),
	.vs_out(vs_out)
);

/////////////////////////////////////////////////////////////////////////////
// Universal Vide Output Interface
//
// HS,HV = direct connect with 47ohm DUMP/safe Register
/////////////////////////////////////////////////////////////////////////////

OBUFT_S_24 VR4(.O(VGA_R[0]),.I(r_out[4]),.T(r_tri[4]));
OBUFT_S_24 VR3(.O(VGA_R[1]),.I(r_out[3]),.T(r_tri[3]));
OBUFT_S_24 VR2(.O(VGA_R[2]),.I(r_out[2]),.T(r_tri[2]));
OBUFT_S_24 VR1(.O(VGA_R[3]),.I(r_out[1]),.T(r_tri[1]));
OBUFT_S_24 VR0(.O(VGA_R[4]),.I(r_out[0]),.T(r_tri[0]));

OBUFT_S_24 VG4(.O(VGA_G[0]),.I(g_out[4]),.T(g_tri[4]));
OBUFT_S_24 VG3(.O(VGA_G[1]),.I(g_out[3]),.T(g_tri[3]));
OBUFT_S_24 VG2(.O(VGA_G[2]),.I(g_out[2]),.T(g_tri[2]));
OBUFT_S_24 VG1(.O(VGA_G[3]),.I(g_out[1]),.T(g_tri[1]));
OBUFT_S_24 VG0(.O(VGA_G[4]),.I(g_out[0]),.T(g_tri[0]));

OBUFT_S_24 VB4(.O(VGA_B[0]),.I(b_out[4]),.T(b_tri[4]));
OBUFT_S_24 VB3(.O(VGA_B[1]),.I(b_out[3]),.T(b_tri[3]));
OBUFT_S_24 VB2(.O(VGA_B[2]),.I(b_out[2]),.T(b_tri[2]));
OBUFT_S_24 VB1(.O(VGA_B[3]),.I(b_out[1]),.T(b_tri[1]));
OBUFT_S_24 VB0(.O(VGA_B[4]),.I(b_out[0]),.T(b_tri[0]));

OBUFT_S_2 HS_DRIVER(.O(n_HSYNC),.I(hs_out),.T(hs_tri));
OBUFT_S_2 VS_DRIVER(.O(n_VSYNC),.I(vs_out),.T(vs_tri));

/////////////////////////////////////////////////////////////////////////////
// SOUND DAC interface
/////////////////////////////////////////////////////////////////////////////

// Sigma Delta :  sampling rate = 37.53KHz
wire dac_out;
dac dac(.DACout(dac_out),.DACin(wave_out[9:2]),.Clk(CLK18_432M),.Reset(~ZRESET));
//dac dac(.DACout(dac_out),.DACin(wave_out),.Clk(CLK36_384M),.Reset(1'b0));

// DAC output PIN assign
OBUF_F_24 DACL (.I(dac_out), .O(DAC_L));
OBUF_F_24 DACR (.I(dac_out), .O(DAC_R));

/////////////////////////////////////////////////////////////////////////////
// test & check ciecuit
/////////////////////////////////////////////////////////////////////////////
`ifdef dualshook
wire   ZMWR;
assign ZMWR = ZMREQ | ZWR;

//PACMAN Erase! (erase=1) 

always @(negedge ZMWR)
begin
  if(wram_cs && ZA==16'h4DA5)begin
    if(ZDO >= 8'h05 && ZDO <= 8'h0E) 
        erase<=1;
    else
        erase<=0;
  end
end

//Demonstration Image(Play=0) or In the Game(play=1)

always @(negedge ZMWR)
begin
  if(wram_cs && ZA==16'h4E02)begin
    if(ZDO == 8'h00) 
        play<=1;
    else
        play<=0;
  end
end

//PACMAN GO TO WARP TONNEL

always @(negedge ZMWR)
begin
  if(wram_cs && ZA==16'h4C1B)begin
    if(ZDO == 8'h8C) 
        tunnel_y<=1;
    else
        tunnel_y<=0;
  end
end
always @(negedge ZMWR)
begin
  if(wram_cs && ZA==16'h4C1A)begin
    if((ZDO >= 8'h00 && ZDO <= 8'h3E)||
        (ZDO >= 8'hCF && ZDO <= 8'hFF)) 
        tunnel_x<=1;
    else
        tunnel_x<=0;
  end
end
`endif
endmodule
